/**
 * Editor component cypress testing
 * Due to lack of possibility to choose row for cursor start
 * we need to create a new blank line to type on in editor
 */
import React from "react";
import { Box } from "@mui/material";
import { Editor, EditorBox } from "../Editor";
import { files } from "../../../mocks/Editor/Editor";

function EditorTemplate({ readOnly = false, fileType }) {
  const options = {
    readOnly,
    filename: fileType,
    theme: "monokai",
    mode: undefined,
    value: files[fileType],
    width: "100%",
    height: "100%"
  };

  return (
    <Box
      height="100vh"
      style={{ margin: "-8px" }}
    >
      <EditorBox>
        <Editor
          name="test-editor"
          {...options}
        />
      </EditorBox>
    </Box>
  );
}

describe("Editor.spec.js", () => {
  const text = "The quick brown fox jumps over the lazy dog.";

  context("Editor, writable", () => {
    Object.keys(files).map((file) => {
      return it(`Editor, writable, ${file}`, () => {
        cy.mount(<EditorTemplate fileType={file} />);
        cy.get("#test-editor")
          // We need to make a clear row to be able to find it later
          .type(`{pageUp}${text}{enter}`);
        cy.contains(text).should("exist");
      });
    });
  });

  context("Editor, not writable", () => {
    Object.keys(files).map((file) => {
      return it(`Editor, not writable, ${file}`, () => {
        cy.mount(
          <EditorTemplate
            fileType={file}
            readOnly
          />
        );
        cy.get("#test-editor")
          // We need to make a clear row to be able to find it later
          .type(`{pageUp}${text}{enter}`)
          .should("have.value", "");
      });
    });
  });
});
