/**
 * Global App Bar
 * component utilising:
 * Mui Toolbar: https://mui.com/material-ui/api/toolbar/#main-content
 * AppBox, BodyBox, ContentBox, HeaderBox, ButtonAppBar, MenuDrawer
 */
import { styled, Toolbar } from "@mui/material";
import React, { useCallback, useState, useEffect } from "react";
import { AppBox, BodyBox, ContentBox, HeaderBox } from "../../common/AppLayout";
import { ButtonAppBar } from "../ButtonAppBar";
import { MenuDrawer } from "../MenuDrawer";
import { GlobalAppBarContext } from "../../../contexts/GlobalAppBarContext";

const defaultStyle = ({ theme, ...props }) => ({
  display: "flex",
  "& > .CeuiGlobalAppBar-Header": {
    zIndex: theme.zIndex.drawer + 1
  },
  "& > .CeuiGlobalAppBar-Body > .CeuiGlobalAppBar-ContentBox": {
    padding: theme.spacing(3),
    alignItems: "center"
  },
  "& > .CeuiGlobalAppBar-Body > .CeuiGlobalAppBar-MenuBox": {
    flex: "0 1 auto"
  }
});

/**
 * GlobalAppBar
 * TODO
 * the props for this component will have to be
 * more properly defined with props-types
 */
export const GlobalAppBar = styled(
  ({
    defaultHomeButton,
    defaultActionButton,
    defaultIconMenuButton,
    defaultHelpButton,
    defaultTitle,
    defaultOpen,
    menuItems,
    widthOpen,
    widthClosed,
    children,
    className,
    getDrawerOpen,
    ...delegated
  }) => {
    const [homeButton, setHomeButton] = useState(defaultHomeButton);
    const [actionButton, setActionButton] = useState(defaultActionButton);
    const [iconMenuButton, setIconMenuButton] = useState(defaultIconMenuButton);
    const [helpButton, setHelpButton] = useState(defaultHelpButton);
    const [title, setTitle] = useState(defaultTitle);
    const setBrowserAndLocalTitle = (newTitle) => {
      document.title = newTitle;
      setTitle(newTitle);
    };

    const [value] = useState({
      setHomeButton,
      setActionButton,
      setIconMenuButton,
      setHelpButton,
      setTitle: setBrowserAndLocalTitle
    });
    const [drawerOpen, setDrawerOpen] = useState(defaultOpen);

    const toggleDrawer = useCallback(() => {
      setDrawerOpen(!drawerOpen);
    }, [drawerOpen]);

    useEffect(() => {
      getDrawerOpen(drawerOpen);
    }, [drawerOpen, getDrawerOpen]);

    return (
      <AppBox
        className={`${className} CeuiGlobalAppBar`}
        {...delegated}
      >
        <HeaderBox className="CeuiGlobalAppBar-Header">
          <ButtonAppBar
            {...{ homeButton, actionButton, iconMenuButton, helpButton, title }}
          />
          <Toolbar className="CeuiGlobalAppBar-Spacer" />
        </HeaderBox>
        <BodyBox className="CeuiGlobalAppBar-Body">
          <ContentBox className="CeuiGlobalAppBar-MenuBox">
            <MenuDrawer
              {...{ open: drawerOpen, toggleDrawer, widthOpen, widthClosed }}
            >
              {menuItems}
            </MenuDrawer>
          </ContentBox>
          <ContentBox
            className="CeuiGlobalAppBar-ContentBox"
            sx={{
              width: "100%"
            }}
          >
            <GlobalAppBarContext.Provider value={value}>
              {children}
            </GlobalAppBarContext.Provider>
          </ContentBox>
        </BodyBox>
      </AppBox>
    );
  },
  {
    name: "GlobalAppBar",
    slot: "Root"
  }
)(defaultStyle);
