import { Box, Typography, styled } from "@mui/material";
import { oneOfType, string, element } from "prop-types";
import React from "react";
import { ExternalLink } from "../../common/Link/ExternalLink";

const StyledBox = styled(Box)`
  margin-bottom: 1rem;
`;

export const Help = styled(
  ({ summary, docsHref, supportHref, version, versionHref, className }) => {
    return (
      <Box className={className}>
        <StyledBox>
          <Typography variant="h2">About</Typography>
          {typeof summary === "string" ? (
            <Typography>{summary}</Typography>
          ) : (
            summary
          )}
        </StyledBox>
        <StyledBox>
          <Typography variant="h2">Support</Typography>
          <Typography
            display="flex"
            gap={0.5}
          >
            Want more information about this app?
            <ExternalLink
              href={docsHref}
              label="Visit the documentation to learn more"
            >
              Read the docs
            </ExternalLink>
          </Typography>
          <Typography
            display="flex"
            gap={0.5}
          >
            Experiencing issues or want to suggest an improvement?
            <ExternalLink
              href={supportHref}
              label="Report an issue or improvement with the support desk"
            >
              Contact support
            </ExternalLink>
          </Typography>
        </StyledBox>
        <StyledBox>
          <Typography
            fontWeight="bold"
            display="flex"
            gap={0.5}
          >
            Version:
            {versionHref ? (
              <ExternalLink
                href={versionHref}
                label={`version ${version}`}
              >
                {version}
              </ExternalLink>
            ) : (
              <Typography>{version}</Typography>
            )}
          </Typography>
        </StyledBox>
      </Box>
    );
  }
)({});

Help.propTypes = {
  summary: oneOfType([string, element]),
  docsHref: string,
  supportHref: string,
  version: string
};
