import { rest } from "msw";
import { apiHandlers } from "../mockAPI";

const glue = (requestHandler, delay) => {
  return (req, res, ctx) => {
    const d = requestHandler(req);
    const body = ctx.json(d.body || {});
    const status = ctx.status(d.status || 200);
    let cookie;
    // the only cookie the mockAPI will set is the authorization cookie
    const cookieHeader = d.headers?.["Set-Cookie"];
    if (cookieHeader) {
      const [name, value] = cookieHeader.split(";")[0].split("=");
      const opts = cookieHeader.includes("Max-Age")
        ? { expires: new Date(0) }
        : {};
      cookie = ctx.cookie(name, value, opts);
    }
    if (delay) {
      return res(ctx.delay(delay), body, status, cookie);
    } else {
      return res(body, status, cookie);
    }
  };
};

// define our MSW handlers to mock our API endpoints
export const handlers = apiHandlers.map((h) => {
  const restMap = {
    GET: rest.get,
    POST: rest.post
  };
  const method = restMap[h.method];

  const isSpec = h.matcher.toString().includes("spec");
  const delay = h.delay ?? 800;
  return method(h.matcher, glue(h.requestHandler, isSpec ? 0 : delay));
});
