import React, { useContext, useEffect } from "react";
import { Link } from "@mui/material";
import { APIProvider, useAPIMethod, usePagination } from "../../../hooks/API";
import { usePolling } from "../../../hooks/Polling";
import { Table } from "../../../components/common/Table";
import RootPaper from "../../../components/common/container/RootPaper";
import GlobalAppBar from "../../../components/navigation/GlobalAppBar";
import { columns, apiOptions, apiContext } from "../../utils/common-data";
import { sbDisabledArg } from "../../utils/common-args";
import { handlers } from "../../../mocks/msw/handlers";
import { rest } from "msw";

export default {
  title: "Common/Table"
};

const POLL_INTERVAL = 10000;

const Harness = ({ children }) => (
  <APIProvider
    context={apiContext}
    options={apiOptions}
  >
    <GlobalAppBar
      getDrawerOpen={() => {}}
      defaultTitle="Tables"
    >
      <RootPaper>{children}</RootPaper>
    </GlobalAppBar>
  </APIProvider>
);

const PaginatableTable = (args) => {
  const { storybookEnablePagination, ...rest } = args;

  const client = useContext(apiContext);
  const { pagination, setPagination } = usePagination({
    rowsPerPageOptions: [5, 10, 20, 50]
  });
  const {
    value,
    loading,
    abort,
    wrapper: getHosts
  } = useAPIMethod({
    fcn: client?.apis.hosts.list_hosts,
    params: storybookEnablePagination ? pagination : null
  });

  usePolling(getHosts, loading, POLL_INTERVAL, abort, false);

  useEffect(
    () => setPagination({ totalCount: value?.totalCount ?? 0 }),
    [setPagination, value?.totalCount]
  );

  const rows = value?.hostList?.map((row) => ({
    ...row,
    name: <Link>{row.name}</Link>
  }));

  let tableParams = { rows, loading };
  if (storybookEnablePagination) {
    tableParams = { ...tableParams, onPage: setPagination, pagination };
  }
  tableParams = { ...tableParams, ...rest };

  return <Table {...tableParams} />;
};

const Template = (args) => {
  return (
    <Harness>
      <PaginatableTable {...args} />
    </Harness>
  );
};

export const Default = (args) => <Template {...args} />;
Default.args = {
  columns,
  striped: true,
  loadingRowCount: 5,
  enableFilterToolbar: false,
  storybookEnablePagination: false // This is specific to these stories!
};
Default.argTypes = {
  columns: sbDisabledArg,
  rows: sbDisabledArg,
  rowClassName: sbDisabledArg
};

export const Empty = (args) => <Template {...args} />;
Empty.args = {
  ...Default.args
};
Empty.argTypes = {
  ...Default.argTypes
};
Empty.parameters = {
  msw: {
    handlers: [
      rest.get("*/hosts", (req, res, ctx) => res(ctx.json([]))),
      ...handlers
    ]
  }
};
export const InitialLoad = (args) => <Template {...args} />;
InitialLoad.args = {
  ...Default.args,
  loading: true
};
InitialLoad.argTypes = {
  ...Default.argTypes,
  loading: sbDisabledArg
};
InitialLoad.parameters = { ...Empty.parameters };
